@@ -26,6 +26,108 @@
 
 namespace // anonymous
 {
+xmlpp::Node* _convert_node(xmlNode* node)
+{
+  xmlpp::Node* res = nullptr;
+  if (node)
+  {
+    xmlpp::Node::create_wrapper(node);
+    res = static_cast<xmlpp::Node*>(node->_private);
+  }
+  return res;
+}
+
+// Common part of const and non-const get_children()
+template <typename Tlist>
+Tlist get_children_common(const Glib::ustring& name, xmlNode* child)
+{
+  Tlist children;
+
+  while (child)
+  {
+    if (name.empty() || name == (const char*)child->name)
+      children.push_back(_convert_node(child));
+
+    child = child->next;
+  }
+  return children;
+}
+
+// Common part of all overloaded xmlpp::Node::find() methods.
+template <typename Tvector>
+Tvector find_common(const Glib::ustring& xpath,
+  const xmlpp::Node::PrefixNsMap* namespaces, xmlNode* node)
+{
+  auto ctxt = xmlXPathNewContext(node->doc);
+  if (!ctxt)
+    throw xmlpp::internal_error("Could not create XPath context for " + xpath);
+  ctxt->node = node;
+
+  if (namespaces)
+  {
+    for (xmlpp::Node::PrefixNsMap::const_iterator it = namespaces->begin();
+         it != namespaces->end(); ++it)
+      xmlXPathRegisterNs(ctxt,
+        reinterpret_cast<const xmlChar*>(it->first.c_str()),
+        reinterpret_cast<const xmlChar*>(it->second.c_str()));
+  }
+
+  auto result = xmlXPathEval((const xmlChar*)xpath.c_str(), ctxt);
+
+  if (!result)
+  {
+    xmlXPathFreeContext(ctxt);
+
+    throw xmlpp::exception("Invalid XPath: " + xpath);
+  }
+
+  if (result->type != XPATH_NODESET)
+  {
+    xmlXPathFreeObject(result);
+    xmlXPathFreeContext(ctxt);
+
+    throw xmlpp::internal_error("Only nodeset result types are supported.");
+  }
+
+  auto nodeset = result->nodesetval;
+  Tvector nodes;
+  if (nodeset && !xmlXPathNodeSetIsEmpty(nodeset))
+  {
+    const int count = xmlXPathNodeSetGetLength(nodeset);
+    nodes.reserve(count);
+    for (int i = 0; i != count; ++i)
+    {
+      auto cnode = xmlXPathNodeSetItem(nodeset, i);
+      if (!cnode)
+      {
+        std::cerr << "Node::find(): The xmlNode was null." << std::endl;
+        continue;
+      }
+
+      if (cnode->type == XML_NAMESPACE_DECL)
+      {
+        //In this case we would cast it to a xmlNs*,
+        //but this C++ method only returns Nodes.
+        std::cerr << "Node::find(): Ignoring an xmlNs object." << std::endl;
+        continue;
+      }
+      
+      //TODO: Check for other cnode->type values?
+  
+      nodes.push_back(_convert_node(cnode));
+    }
+  }
+  else
+  {
+    // return empty set
+  }
+
+  xmlXPathFreeObject(result);
+  xmlXPathFreeContext(ctxt);
+
+  return nodes;
+}
+
 // Common part of xmlpp::Node::eval_to_[boolean|number|string]
 xmlXPathObject* eval_common(const Glib::ustring& xpath,
   const xmlpp::Node::PrefixNsMap* namespaces,
@@ -111,18 +213,6 @@ Glib::ustring eval_common_to_string(const Glib::ustring& xpath,
   return Glib::ustring();
 }
 
-// Common part of all add_child*() methods.
-xmlpp::Element* add_child_common(const Glib::ustring& name, xmlNode* child, xmlNode* node)
-{
-  if (!node)
-  {
-    xmlFreeNode(child);
-    throw xmlpp::internal_error("Could not add child element node " + name);
-  }
-  xmlpp::Node::create_wrapper(node);
-  return static_cast<xmlpp::Element*>(node->_private);
-}
-
 } // anonymous namespace
 
 namespace xmlpp
@@ -182,17 +272,6 @@ Node* Node::get_previous_sibling()
   return static_cast<Node*>(cobj()->prev->_private);
 }
 
-static Node* _convert_node(xmlNode* node)
-{
-  Node* res = nullptr;
-  if(node)
-  {
-    Node::create_wrapper(node);
-    res = static_cast<Node*>(node->_private);
-  }
-  return res;
-}
-
 Node* Node::get_first_child(const Glib::ustring& name)
 {
   auto child = impl_->children;
@@ -216,156 +295,24 @@ const Node* Node::get_first_child(const Glib::ustring& name) const
 
 Node::NodeList Node::get_children(const Glib::ustring& name)
 {
-   auto child = impl_->children;
-   if(!child)
-     return NodeList();
-
-   NodeList children;
-   do
-   {
-      if(name.empty() || name == (const char*)child->name)
-        children.push_back(_convert_node(child));
-   }
-   while((child = child->next));
-   
-   return children;
+  return get_children_common<NodeList>(name, impl_->children);
 }
 
-const Node::NodeList Node::get_children(const Glib::ustring& name) const
+Node::const_NodeList Node::get_children(const Glib::ustring& name) const
 {
-  return const_cast<Node*>(this)->get_children(name);
+  return get_children_common<const_NodeList>(name, impl_->children);
 }
 
-Element* Node::add_child(const Glib::ustring& name,
-                         const Glib::ustring& ns_prefix)
+//static
+void Node::remove_node(Node* node)
 {
-  auto child = create_new_child_node(name, ns_prefix);
-  auto node = xmlAddChild(impl_, child);
-  return add_child_common(name, child, node);
-}
-
-Element* Node::add_child(xmlpp::Node* previous_sibling, 
-                         const Glib::ustring& name,
-                         const Glib::ustring& ns_prefix)
-{
-  if (!previous_sibling)
-    return 0;
-
-  auto child = create_new_child_node(name, ns_prefix);
-  auto node = xmlAddNextSibling(previous_sibling->cobj(), child);
-  return add_child_common(name, child, node);
-}
-
-Element* Node::add_child_before(xmlpp::Node* next_sibling, 
-                         const Glib::ustring& name,
-                         const Glib::ustring& ns_prefix)
-{
-  if (!next_sibling)
-    return 0;
-
-  auto child = create_new_child_node(name, ns_prefix);
-  auto node = xmlAddPrevSibling(next_sibling->cobj(), child);
-  return add_child_common(name, child, node);
-}
-
-Element* Node::add_child_with_new_ns(const Glib::ustring& name,
-  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
-{
-  auto child = create_new_child_node_with_new_ns(name, ns_uri, ns_prefix);
-  auto node = xmlAddChild(impl_, child);
-  return add_child_common(name, child, node);
-}
-
-Element* Node::add_child_with_new_ns(xmlpp::Node* previous_sibling,
-  const Glib::ustring& name,
-  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
-{
-  if (!previous_sibling)
-    return 0;
-
-  auto child = create_new_child_node_with_new_ns(name, ns_uri, ns_prefix);
-  auto node = xmlAddNextSibling(previous_sibling->cobj(), child);
-  return add_child_common(name, child, node);
-}
-
-Element* Node::add_child_before_with_new_ns(xmlpp::Node* next_sibling,
-  const Glib::ustring& name,
-  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
-{
-  if (!next_sibling)
-    return 0;
-
-  auto child = create_new_child_node_with_new_ns(name, ns_uri, ns_prefix);
-  auto node = xmlAddPrevSibling(next_sibling->cobj(), child);
-  return add_child_common(name, child, node);
-}
-
-_xmlNode* Node::create_new_child_node(const Glib::ustring& name, const Glib::ustring& ns_prefix)
-{
-   xmlNs* ns = nullptr;
-
-   if(impl_->type != XML_ELEMENT_NODE)
-   {
-     throw internal_error("You can only add child nodes to element nodes");
-   }
-
-   if(ns_prefix.empty())
-   {
-     //Retrieve default namespace if it exists
-     ns = xmlSearchNs(impl_->doc, impl_, 0);
-   }
-   else
-   {
-     //Use the existing namespace if one exists:
-     ns = xmlSearchNs(impl_->doc, impl_, (const xmlChar*)ns_prefix.c_str());
-     if (!ns)
-     {
-       throw exception("The namespace prefix (" + ns_prefix + ") has not been declared.");
-     }
-   }
-
-   return xmlNewNode(ns, (const xmlChar*)name.c_str());
-}
-
-_xmlNode* Node::create_new_child_node_with_new_ns(const Glib::ustring& name,
-  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
-{
-  if (impl_->type != XML_ELEMENT_NODE)
-    throw internal_error("You can only add child nodes to element nodes.");
-
-  auto child = xmlNewNode(0, (const xmlChar*)name.c_str());
-  if (!child)
-    throw internal_error("Could not create new element node.");
-
-  auto ns = xmlNewNs(child, (const xmlChar*)(ns_uri.empty() ? 0 : ns_uri.c_str()),
-                       (const xmlChar*)(ns_prefix.empty() ? 0 : ns_prefix.c_str()) );
-  // xmlNewNs() does not create a namespace node for the predefined xml prefix.
-  // It's usually defined in the document and not in any specific node.
-  if (!ns && ns_prefix == "xml")
-  {
-    ns = xmlSearchNs(impl_->doc, impl_, (const xmlChar*)ns_prefix.c_str());
-    if (ns && (ns_uri != (ns->href ? (const char*)ns->href : "")))
-      ns = nullptr;
-  }
-  if (!ns)
-  {
-    xmlFreeNode(child);
-    throw internal_error("Could not create new namespace node.");
-  }
-
-  xmlSetNs(child, ns);
-
-  return child;
-}
-
-void Node::remove_child(Node* node)
-{
-  //TODO: Allow a node to be removed without deleting it, to allow it to be moved?
+  //TODO: Allow a node to be disconnected from its parent without deleting it,
+  // to allow it to be moved?
   //This would require a more complex memory management API.
   if (!node)
     return;
   auto cnode = node->cobj();
-  Node::free_wrappers(cnode); //This delete the C++ node (not this) itself.
+  Node::free_wrappers(cnode); // This deletes the C++ node.
   xmlUnlinkNode(cnode);
   xmlFreeNode(cnode);
 }
@@ -447,91 +394,24 @@ Glib::ustring Node::get_path() const
   return retn;
 }
 
-static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
+Node::NodeSet Node::find(const Glib::ustring& xpath)
 {
-  auto result = xmlXPathEval((const xmlChar*)xpath.c_str(), ctxt);
-
-  if(!result)
-  {
-    xmlXPathFreeContext(ctxt);
-
-    throw exception("Invalid XPath: " + xpath);
-  }
-
-  if(result->type != XPATH_NODESET)
-  {
-    xmlXPathFreeObject(result);
-    xmlXPathFreeContext(ctxt);
-
-    throw internal_error("Only nodeset result types are supported.");
-  }
-
-  auto nodeset = result->nodesetval;
-  NodeSet nodes;
-  if( nodeset && !xmlXPathNodeSetIsEmpty(nodeset))
-  {
-    const int count = xmlXPathNodeSetGetLength(nodeset);
-    nodes.reserve(count);
-    for (int i = 0; i != count; ++i)
-    {
-      auto cnode = xmlXPathNodeSetItem(nodeset, i);
-      if(!cnode)
-      {
-        std::cerr << "Node::find_impl: The xmlNode was null." << std::endl;
-        continue;
-      }
-
-      if(cnode->type == XML_NAMESPACE_DECL)
-      {
-        //In this case we would cast it to a xmlNs*,
-        //but this C++ method only returns Nodes.
-        std::cerr << "Node::find_impl: ignoring an xmlNs object." << std::endl;
-        continue;
-      }
-      
-      //TODO: Check for other cnode->type values?
-  
-      Node::create_wrapper(cnode);
-      auto cppNode = static_cast<Node*>(cnode->_private);
-      nodes.push_back(cppNode);
-    }
-  }
-  else
-  {
-    // return empty set
-  }
-
-  xmlXPathFreeObject(result);
-  xmlXPathFreeContext(ctxt);
-
-  return nodes;
+  return find_common<NodeSet>(xpath, 0, impl_);
 }
 
-NodeSet Node::find(const Glib::ustring& xpath) const
+Node::const_NodeSet Node::find(const Glib::ustring& xpath) const
 {
-  auto ctxt = xmlXPathNewContext(impl_->doc);
-  if (!ctxt)
-    throw internal_error("Could not create XPath context for " + xpath);
-  ctxt->node = impl_;
-  
-  return find_impl(ctxt, xpath);
+  return find_common<const_NodeSet>(xpath, 0, impl_);
 }
 
-NodeSet Node::find(const Glib::ustring& xpath,
-		   const PrefixNsMap& namespaces) const
+Node::NodeSet Node::find(const Glib::ustring& xpath, const PrefixNsMap& namespaces)
 {
-  auto ctxt = xmlXPathNewContext(impl_->doc);
-  if (!ctxt)
-    throw internal_error("Could not create XPath context for " + xpath);
-  ctxt->node = impl_;
+  return find_common<NodeSet>(xpath, &namespaces, impl_);
+}
 
-  for (PrefixNsMap::const_iterator it=namespaces.begin();
-       it != namespaces.end(); it++)
-    xmlXPathRegisterNs(ctxt,
-		       reinterpret_cast<const xmlChar*>(it->first.c_str()),
-		       reinterpret_cast<const xmlChar*>(it->second.c_str()));
-
-  return find_impl(ctxt, xpath);
+Node::const_NodeSet Node::find(const Glib::ustring& xpath, const PrefixNsMap& namespaces) const
+{
+  return find_common<const_NodeSet>(xpath, &namespaces, impl_);
 }
 
 bool Node::eval_to_boolean(const Glib::ustring& xpath, XPathResultType* result_type) const
@@ -776,5 +656,4 @@ void Node::free_wrappers(xmlNode* node)
     free_wrappers(reinterpret_cast<xmlNode*>(attr));
 }
 
-
 } //namespace xmlpp
