@@ -42,7 +42,7 @@ RelaxNGValidator::RelaxNGValidator()
 {
 }
 
-RelaxNGValidator::RelaxNGValidator(const Glib::ustring& filename)
+RelaxNGValidator::RelaxNGValidator(const std::string& filename)
 : pimpl_(new Impl)
 {
   parse_file(filename);
@@ -65,7 +65,7 @@ RelaxNGValidator::~RelaxNGValidator()
   release_underlying();
 }
 
-void RelaxNGValidator::parse_file(const Glib::ustring& filename)
+void RelaxNGValidator::parse_file(const std::string& filename)
 {
   set_schema(new RelaxNGSchema(filename), true);
 }
@@ -117,16 +117,16 @@ const RelaxNGSchema* RelaxNGValidator::get_schema() const
   return pimpl_->schema;
 }
 
-RelaxNGValidator::operator const void*() const
+RelaxNGValidator::operator bool() const noexcept
 {
-  return reinterpret_cast<const void*>(pimpl_->schema && pimpl_->schema->cobj());
+  return pimpl_->schema && pimpl_->schema->cobj();
 }
 
 
-void RelaxNGValidator::initialize_valid()
+void RelaxNGValidator::initialize_context()
 {
   xmlRelaxNGSetValidErrors(pimpl_->context, &callback_validity_error, &callback_validity_warning, this);
-  SchemaValidatorBase::initialize_valid();
+  SchemaValidatorBase::initialize_context();
 }
 
 void RelaxNGValidator::validate(const Document* document)
@@ -145,7 +145,7 @@ void RelaxNGValidator::validate(const Document* document)
     throw internal_error("RelaxNGValidator::validate(): Could not create validating context");
 
   xmlResetLastError();
-  initialize_valid();
+  initialize_context();
 
   const int res = xmlRelaxNGValidateDoc(pimpl_->context, const_cast<xmlDoc*>(document->cobj()));
   if (res != 0)
@@ -159,7 +159,7 @@ void RelaxNGValidator::validate(const Document* document)
   }
 }
 
-void RelaxNGValidator::validate(const Glib::ustring& filename)
+void RelaxNGValidator::validate(const std::string& filename)
 {
   // There is no xmlRelaxNGValidateFile().
   DomParser parser(filename);
