@@ -39,7 +39,7 @@ XsdValidator::XsdValidator()
 {
 }
 
-XsdValidator::XsdValidator(const Glib::ustring& filename)
+XsdValidator::XsdValidator(const std::string& filename)
 : pimpl_(new Impl)
 {
   parse_file(filename);
@@ -62,7 +62,7 @@ XsdValidator::~XsdValidator()
   release_underlying();
 }
 
-void XsdValidator::parse_file(const Glib::ustring& filename)
+void XsdValidator::parse_file(const std::string& filename)
 {
   set_schema(new XsdSchema(filename), true);
 }
@@ -114,23 +114,23 @@ const XsdSchema* XsdValidator::get_schema() const
   return pimpl_->schema;
 }
 
-XsdValidator::operator const void*() const
+XsdValidator::operator bool() const noexcept
 {
-  return reinterpret_cast<const void*>(pimpl_->schema && pimpl_->schema->cobj());
+  return pimpl_->schema && pimpl_->schema->cobj();
 }
 
 
-void XsdValidator::initialize_valid()
+void XsdValidator::initialize_context()
 {
   xmlSchemaSetValidErrors(pimpl_->context, &callback_validity_error, &callback_validity_warning, this);
-  SchemaValidatorBase::initialize_valid();
+  SchemaValidatorBase::initialize_context();
 }
 
 
 void XsdValidator::validate(const Document* document)
 {
   if (!document)
-    throw internal_error("XsdValidator::validate(): document must not be nullptr.");
+    throw internal_error("XsdValidator::validate(): document must not be 0.");
 
   if (!*this)
     throw internal_error("XsdValidator::validate(): Must have a schema to validate document");
@@ -143,7 +143,7 @@ void XsdValidator::validate(const Document* document)
     throw internal_error("XsdValidator::validate(): Could not create validating context");
 
   xmlResetLastError();
-  initialize_valid();
+  initialize_context();
 
   const int res = xmlSchemaValidateDoc(pimpl_->context, const_cast<xmlDoc*>(document->cobj()));
   if (res != 0)
@@ -157,7 +157,7 @@ void XsdValidator::validate(const Document* document)
   }
 }
 
-void XsdValidator::validate(const Glib::ustring& filename)
+void XsdValidator::validate(const std::string& filename)
 {
   if (!*this)
     throw internal_error("XsdValidator::validate(): Must have a schema to validate file.");
@@ -170,7 +170,7 @@ void XsdValidator::validate(const Glib::ustring& filename)
     throw internal_error("XsdValidator::validate(): Could not create validating context");
 
   xmlResetLastError();
-  initialize_valid();
+  initialize_context();
 
   const int res = xmlSchemaValidateFile(pimpl_->context, filename.c_str(), 0);
   if (res != 0)
